@@ -23,10 +23,10 @@
 #'
 #' @return For \strong{cv}: a 'conditionVariable' object.
 #'
-#'     For \strong{wait} and \strong{until}: (invisibly) logical TRUE, or else
-#'     FALSE if a flag has been set.
+#'     For \strong{wait}: (invisibly) logical TRUE, or else FALSE if a flag has
+#'     been set.
 #'
-#'     For \strong{.until}: (invisibly) logical TRUE if signalled, or else FALSE
+#'     For \strong{until}: (invisibly) logical TRUE if signalled, or else FALSE
 #'     if the timeout was reached.
 #'
 #'     For \strong{cv_value}: integer value of the condition variable.
@@ -79,28 +79,37 @@ cv <- function() .Call(rnng_cv_alloc)
 #' Condition Variables - Wait
 #'
 #' \code{wait} waits on a condition being signalled by completion of an
-#'     asynchronous receive or pipe event.
+#'     asynchronous receive or pipe event. \cr \code{wait_} is a variant that
+#'     allows user interrupts, suitable for interactive use.
 #'
 #' @param cv a 'conditionVariable' object.
 #'
 #' @examples
 #' # wait(cv) # uncommenting will block until the cv is signalled
+#' # wait_(cv) # block until the cv is signalled or interrupted
 #'
 #' @rdname cv
 #' @export
 #'
 wait <- function(cv) invisible(.Call(rnng_cv_wait, cv))
 
+#' @rdname cv
+#' @export
+#'
+wait_ <- function(cv) invisible(.Call(rnng_cv_wait_safe, cv))
+
 #' Condition Variables - Until
 #'
 #' \code{until} waits until a future time on a condition being signalled by
-#'     completion of an asynchronous receive or pipe event.
+#'     completion of an asynchronous receive or pipe event. \cr \code{until_} is
+#'     a variant that allows user interrupts, suitable for interactive use.
 #'
 #' @param msec maximum time in milliseconds to wait for the condition variable
 #'     to be signalled.
 #'
 #' @examples
 #' until(cv, 10L)
+#' until_(cv, 10L)
 #'
 #' @rdname cv
 #' @export
@@ -110,7 +119,7 @@ until <- function(cv, msec) invisible(.Call(rnng_cv_until, cv, msec))
 #' @rdname cv
 #' @export
 #'
-.until <- function(cv, msec) invisible(.Call(rnng_cv_until2, cv, msec))
+until_ <- function(cv, msec) invisible(.Call(rnng_cv_until_safe, cv, msec))
 
 #' Condition Variables - Value
 #'
@@ -150,6 +159,19 @@ cv_reset <- function(cv) invisible(.Call(rnng_cv_reset, cv))
 #'
 cv_signal <- function(cv) invisible(.Call(rnng_cv_signal, cv))
 
+#' Condition Variables - Until
+#'
+#' This function is identical to \code{\link{until}} and deprecated.
+#'
+#' @inheritParams until
+#'
+#' @return (invisibly) logical TRUE if signalled, or else FALSE if the timeout
+#'     was reached.
+#'
+#' @export
+#'
+.until <- until
+
 #' Pipe Notify
 #'
 #' Signals a 'conditionVariable' whenever pipes (individual connections) are
@@ -160,11 +182,11 @@ cv_signal <- function(cv) invisible(.Call(rnng_cv_signal, cv))
 #' @param cv2 [default NULL] optionally, if specified, a second
 #'     'conditionVariable' to signal. Note that this cv is signalled
 #'     sequentially after the first condition variable.
-#' @param add [default TRUE] logical value whether to signal when a pipe is
+#' @param add [default FALSE] logical value whether to signal when a pipe is
 #'     added.
-#' @param remove [default TRUE] logical value whether to signal when a pipe is
+#' @param remove [default FALSE] logical value whether to signal when a pipe is
 #'     removed.
-#' @param flag [default TRUE] logical value whether to also set a flag in the
+#' @param flag [default FALSE] logical value whether to also set a flag in the
 #'     'conditionVariable'. This can help distinguish between different types of
 #'     signal, and causes any subsequent \code{\link{wait}} to return FALSE
 #'     instead of TRUE.
@@ -202,7 +224,7 @@ cv_signal <- function(cv) invisible(.Call(rnng_cv_signal, cv))
 #'
 #' @export
 #'
-pipe_notify <- function(socket, cv, cv2 = NULL, add = TRUE, remove = TRUE, flag = TRUE)
+pipe_notify <- function(socket, cv, cv2 = NULL, add = FALSE, remove = FALSE, flag = FALSE)
   invisible(.Call(rnng_pipe_notify, socket, cv, cv2, add, remove, flag))
 
 #' Lock / Unlock a Socket
@@ -249,3 +271,37 @@ lock <- function(socket, cv = NULL) invisible(.Call(rnng_socket_lock, socket, cv
 #' @export
 #'
 unlock <- function(socket) invisible(.Call(rnng_socket_unlock, socket))
+
+#' Signal Forwarder
+#'
+#' Forwards signals from one 'conditionVariable' to another.
+#'
+#' @param cv a 'conditionVariable' object, from which to forward the signal.
+#' @param cv2 a 'conditionVariable' object, to which the signal is forwarded.
+#'
+#' @return Invisibly, 'cv2'.
+#'
+#' @details The condition value of 'cv' is initially reset to zero when this
+#'     operator returns. Only one forwarder can be active on a 'cv' at any given
+#'     time, and assigning a new forwarding target cancels any currently
+#'     existing forwarding.
+#'
+#'     Changes in the condition value of 'cv' are forwarded to 'cv2', but only
+#'     on each occassion 'cv' is signalled. This means that waiting on 'cv'
+#'     will cause a temporary divergence between the actual condition value of
+#'     'cv' and that recorded at 'cv2', until the next time 'cv' is signalled.
+#'
+#' @examples
+#' cva <- cv(); cvb <- cv(); cv1 <- cv(); cv2 <- cv()
+#'
+#' cva %~>% cv1 %~>% cv2
+#' cvb %~>% cv2
+#'
+#' cv_signal(cva)
+#' cv_signal(cvb)
+#' cv_value(cv1)
+#' cv_value(cv2)
+#'
+#' @export
+#'
+`%~>%` <- function(cv, cv2) invisible(.Call(rnng_signal_thread_create, cv, cv2))
