@@ -21,6 +21,10 @@
 
 #include <nng/nng.h>
 
+#if NNG_MAJOR_VERSION == 1 && NNG_MINOR_VERSION < 6
+#define NANONEXT_LEGACY_NNG
+#endif
+
 #ifdef NANONEXT_PROTOCOLS
 #include <nng/protocol/bus0/bus.h>
 #include <nng/protocol/pair0/pair.h>
@@ -38,7 +42,7 @@
 #include <nng/supplemental/http/http.h>
 #include <nng/supplemental/tls/tls.h>
 #include <nng/supplemental/util/platform.h>
-#if NNG_MAJOR_VERSION == 1 && NNG_MINOR_VERSION < 6
+#ifdef NANONEXT_LEGACY_NNG
 extern nng_mtx *shr_mtx;
 #endif
 
@@ -52,16 +56,6 @@ typedef struct nano_dialer_s {
   nng_tls_config *tls;
 } nano_dialer;
 
-typedef struct nano_stream_listener_s {
-  nng_stream_listener *list;
-  nng_tls_config *tls;
-} nano_stream_listener;
-
-typedef struct nano_stream_dialer_s {
-  nng_stream_dialer *dial;
-  nng_tls_config *tls;
-} nano_stream_dialer;
-
 typedef enum nano_aio_typ {
   SENDAIO,
   RECVAIO,
@@ -78,13 +72,12 @@ typedef struct nano_aio_s {
   void *data;
 } nano_aio;
 
-typedef struct nano_handle_s {
-  nng_url *url;
-  nng_http_client *cli;
-  nng_http_req *req;
-  nng_http_res *res;
-  nng_tls_config *cfg;
-} nano_handle;
+typedef struct nano_cv_s {
+  int condition;
+  uint8_t flag;
+  nng_mtx *mtx;
+  nng_cv *cv;
+} nano_cv;
 
 #endif
 
@@ -110,7 +103,7 @@ typedef struct nano_handle_s {
 
 #ifdef NANONEXT_KEYCERT
 #include <mbedtls/version.h>
-#if MBEDTLS_VERSION_MAJOR == 2
+#if MBEDTLS_VERSION_MAJOR < 3
 #include <mbedtls/config.h>
 #endif
 #include <mbedtls/platform.h>
@@ -134,18 +127,17 @@ typedef struct nano_handle_s {
 
 #define ERROR_OUT(xc) Rf_error("%d | %s", xc, nng_strerror(xc))
 #define ERROR_RET(xc) { Rf_warning("%d | %s", xc, nng_strerror(xc)); return mk_error(xc); }
-#define NANONEXT_INIT_BUFSIZE 16384
+#define NANONEXT_INIT_BUFSIZE 8192
 #define NANONEXT_SERIAL_VER 3
-#define NANONEXT_INT_STRLEN 12
-#define NANONEXT_ACK_MS 100
+#define NANONEXT_LD_STRLEN 21
 #define NANO_ALLOC(x, sz)                                      \
   (x)->buf = R_Calloc(sz, unsigned char);                      \
-  (x)->len = (R_xlen_t) sz;                                    \
+  (x)->len = sz;                                    \
   (x)->cur = 0
 #define NANO_INIT(x, ptr, sz)                                  \
   (x)->buf = ptr;                                              \
   (x)->len = 0;                                                \
-  (x)->cur = (R_xlen_t) sz
+  (x)->cur = sz
 #define NANO_FREE(x) if (x.len) R_Free(x.buf)
 
 typedef struct nano_buf_s {
@@ -154,22 +146,97 @@ typedef struct nano_buf_s {
   size_t cur;
 } nano_buf;
 
-extern SEXP mk_error(const int);
-extern SEXP mk_error_ncurl(const int);
-extern SEXP nano_decode(unsigned char *, size_t, const int);
-extern void dialer_finalizer(SEXP);
-extern void nano_encode(nano_buf *, SEXP);
-extern void listener_finalizer(SEXP);
-extern int nano_encodes(SEXP);
-extern int nano_matcharg(SEXP);
-extern int nano_matchargs(SEXP);
-extern void nano_serialize(nano_buf *, SEXP);
-extern void nano_serialize_next(nano_buf *, SEXP);
-extern void nano_serialize_xdr(nano_buf *, SEXP);
-extern SEXP nano_unserialize(unsigned char *, const size_t);
-extern SEXP nano_hashToChar(unsigned char *, const size_t);
-extern SEXP rawToChar(unsigned char *, const size_t);
-extern void socket_finalizer(SEXP);
+SEXP mk_error(const int);
+SEXP mk_error_ncurl(const int);
+SEXP nano_decode(unsigned char *, size_t, const int);
+void nano_encode(nano_buf *, SEXP);
+int nano_encodes(SEXP);
+int nano_matcharg(SEXP);
+int nano_matchargs(SEXP);
+void nano_serialize(nano_buf *, SEXP);
+void nano_serialize_next(nano_buf *, SEXP);
+void nano_serialize_xdr(nano_buf *, SEXP);
+SEXP nano_unserialize(unsigned char *, const size_t);
+SEXP nano_hashToChar(unsigned char *, const size_t);
+SEXP rawToChar(unsigned char *, const size_t);
+void dialer_finalizer(SEXP);
+void listener_finalizer(SEXP);
+void socket_finalizer(SEXP);
+
+SEXP rnng_aio_call(SEXP);
+SEXP rnng_aio_get_msg(SEXP);
+SEXP rnng_aio_get_msg2(SEXP);
+SEXP rnng_aio_http(SEXP, SEXP, SEXP);
+SEXP rnng_aio_result(SEXP);
+SEXP rnng_aio_stop(SEXP);
+SEXP rnng_base64dec(SEXP, SEXP);
+SEXP rnng_base64enc(SEXP, SEXP);
+SEXP rnng_clock(void);
+SEXP rnng_close(SEXP);
+SEXP rnng_ctx_close(SEXP);
+SEXP rnng_ctx_create(SEXP);
+SEXP rnng_ctx_open(SEXP);
+SEXP rnng_cv_alloc(void);
+SEXP rnng_cv_recv_aio(SEXP, SEXP, SEXP, SEXP, SEXP, SEXP);
+SEXP rnng_cv_request(SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP);
+SEXP rnng_cv_reset(SEXP);
+SEXP rnng_cv_signal(SEXP);
+SEXP rnng_cv_until(SEXP, SEXP);
+SEXP rnng_cv_until_safe(SEXP, SEXP);
+SEXP rnng_cv_value(SEXP);
+SEXP rnng_cv_wait(SEXP);
+SEXP rnng_cv_wait_safe(SEXP);
+SEXP rnng_dial(SEXP, SEXP, SEXP, SEXP, SEXP);
+SEXP rnng_dialer_close(SEXP);
+SEXP rnng_dialer_start(SEXP, SEXP);
+SEXP rnng_get_opt(SEXP, SEXP);
+SEXP rnng_is_error_value(SEXP);
+SEXP rnng_is_nul_byte(SEXP);
+SEXP rnng_listen(SEXP, SEXP, SEXP, SEXP, SEXP);
+SEXP rnng_listener_close(SEXP);
+SEXP rnng_listener_start(SEXP);
+SEXP rnng_messenger(SEXP);
+SEXP rnng_messenger_thread_create(SEXP);
+SEXP rnng_ncurl(SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP);
+SEXP rnng_ncurl_aio(SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP);
+SEXP rnng_ncurl_session(SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP);
+SEXP rnng_ncurl_session_close(SEXP);
+SEXP rnng_ncurl_transact(SEXP);
+SEXP rnng_next_config(SEXP, SEXP);
+SEXP rnng_pipe_notify(SEXP, SEXP, SEXP, SEXP, SEXP, SEXP);
+SEXP rnng_protocol_open(SEXP, SEXP);
+SEXP rnng_random(SEXP, SEXP);
+SEXP rnng_reap(SEXP);
+SEXP rnng_recv(SEXP, SEXP, SEXP, SEXP);
+SEXP rnng_recv_aio(SEXP, SEXP, SEXP, SEXP, SEXP);
+SEXP rnng_request(SEXP, SEXP, SEXP, SEXP, SEXP, SEXP);
+SEXP rnng_send(SEXP, SEXP, SEXP, SEXP);
+SEXP rnng_send_aio(SEXP, SEXP, SEXP, SEXP, SEXP);
+SEXP rnng_set_opt(SEXP, SEXP, SEXP);
+SEXP rnng_sha1(SEXP, SEXP, SEXP);
+SEXP rnng_sha224(SEXP, SEXP, SEXP);
+SEXP rnng_sha256(SEXP, SEXP, SEXP);
+SEXP rnng_sha384(SEXP, SEXP, SEXP);
+SEXP rnng_sha512(SEXP, SEXP, SEXP);
+SEXP rnng_signal_thread_create(SEXP, SEXP);
+SEXP rnng_sleep(SEXP);
+SEXP rnng_socket_lock(SEXP, SEXP);
+SEXP rnng_socket_unlock(SEXP);
+SEXP rnng_stats_get(SEXP, SEXP);
+SEXP rnng_status_code(SEXP);
+SEXP rnng_stream_close(SEXP);
+SEXP rnng_stream_dial(SEXP, SEXP, SEXP);
+SEXP rnng_stream_listen(SEXP, SEXP, SEXP);
+SEXP rnng_strcat(SEXP, SEXP);
+SEXP rnng_strerror(SEXP);
+SEXP rnng_subscribe(SEXP, SEXP, SEXP);
+SEXP rnng_tls_config(SEXP, SEXP, SEXP, SEXP);
+SEXP rnng_unresolved(SEXP);
+SEXP rnng_unresolved2(SEXP);
+SEXP rnng_url_parse(SEXP);
+SEXP rnng_version(void);
+SEXP rnng_wait_thread_create(SEXP);
+SEXP rnng_write_cert(SEXP, SEXP, SEXP);
 
 extern SEXP nano_AioSymbol;
 extern SEXP nano_ContextSymbol;
@@ -200,86 +267,11 @@ extern SEXP nano_error;
 extern SEXP nano_ncurlAio;
 extern SEXP nano_ncurlSession;
 extern SEXP nano_recvAio;
-extern SEXP nano_refHook;
+extern SEXP nano_refHookIn;
+extern SEXP nano_refHookOut;
 extern SEXP nano_refList;
 extern SEXP nano_sendAio;
 extern SEXP nano_success;
 extern SEXP nano_unresolved;
 
-extern SEXP rnng_aio_call(SEXP);
-extern SEXP rnng_aio_get_msg(SEXP);
-extern SEXP rnng_aio_get_msg2(SEXP);
-extern SEXP rnng_aio_http(SEXP, SEXP, SEXP);
-extern SEXP rnng_aio_result(SEXP);
-extern SEXP rnng_aio_stop(SEXP);
-extern SEXP rnng_base64dec(SEXP, SEXP);
-extern SEXP rnng_base64enc(SEXP, SEXP);
-extern SEXP rnng_clock(void);
-extern SEXP rnng_close(SEXP);
-extern SEXP rnng_ctx_close(SEXP);
-extern SEXP rnng_ctx_create(SEXP);
-extern SEXP rnng_ctx_open(SEXP);
-extern SEXP rnng_cv_alloc(void);
-extern SEXP rnng_cv_recv_aio(SEXP, SEXP, SEXP, SEXP, SEXP, SEXP);
-extern SEXP rnng_cv_request(SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP);
-extern SEXP rnng_cv_reset(SEXP);
-extern SEXP rnng_cv_signal(SEXP);
-extern SEXP rnng_cv_until(SEXP, SEXP);
-extern SEXP rnng_cv_until2(SEXP, SEXP);
-extern SEXP rnng_cv_value(SEXP);
-extern SEXP rnng_cv_wait(SEXP);
-extern SEXP rnng_dial(SEXP, SEXP, SEXP, SEXP, SEXP);
-extern SEXP rnng_dialer_close(SEXP);
-extern SEXP rnng_dialer_start(SEXP, SEXP);
-extern void rnng_fini(void);
-extern SEXP rnng_get_opt(SEXP, SEXP);
-extern SEXP rnng_is_error_value(SEXP);
-extern SEXP rnng_is_nul_byte(SEXP);
-extern SEXP rnng_listen(SEXP, SEXP, SEXP, SEXP, SEXP);
-extern SEXP rnng_listener_close(SEXP);
-extern SEXP rnng_listener_start(SEXP);
-extern SEXP rnng_messenger(SEXP);
-extern SEXP rnng_messenger_thread_create(SEXP);
-extern SEXP rnng_ncurl(SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP);
-extern SEXP rnng_ncurl_aio(SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP);
-extern SEXP rnng_ncurl_session(SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP);
-extern SEXP rnng_ncurl_session_close(SEXP);
-extern SEXP rnng_ncurl_transact(SEXP);
-extern SEXP rnng_next_mode(SEXP, SEXP, SEXP);
-extern SEXP rnng_pipe_notify(SEXP, SEXP, SEXP, SEXP, SEXP, SEXP);
-extern SEXP rnng_protocol_open(SEXP, SEXP);
-extern SEXP rnng_random(SEXP, SEXP);
-extern SEXP rnng_reap(SEXP);
-extern SEXP rnng_recv(SEXP, SEXP, SEXP, SEXP);
-extern SEXP rnng_recv_aio(SEXP, SEXP, SEXP, SEXP, SEXP);
-extern SEXP rnng_request(SEXP, SEXP, SEXP, SEXP, SEXP, SEXP);
-extern SEXP rnng_send(SEXP, SEXP, SEXP, SEXP);
-extern SEXP rnng_send_aio(SEXP, SEXP, SEXP, SEXP, SEXP);
-extern SEXP rnng_set_opt(SEXP, SEXP, SEXP);
-extern SEXP rnng_sha1(SEXP, SEXP, SEXP);
-extern SEXP rnng_sha224(SEXP, SEXP, SEXP);
-extern SEXP rnng_sha256(SEXP, SEXP, SEXP);
-extern SEXP rnng_sha384(SEXP, SEXP, SEXP);
-extern SEXP rnng_sha512(SEXP, SEXP, SEXP);
-extern SEXP rnng_sleep(SEXP);
-extern SEXP rnng_socket_lock(SEXP, SEXP);
-extern SEXP rnng_socket_unlock(SEXP);
-extern SEXP rnng_stats_get(SEXP, SEXP);
-extern SEXP rnng_status_code(SEXP);
-extern SEXP rnng_stream_close(SEXP);
-extern SEXP rnng_stream_dial(SEXP, SEXP, SEXP);
-extern SEXP rnng_stream_listen(SEXP, SEXP, SEXP);
-extern SEXP rnng_strcat(SEXP, SEXP);
-extern SEXP rnng_strerror(SEXP);
-extern SEXP rnng_subscribe(SEXP, SEXP, SEXP);
-extern SEXP rnng_tls_config(SEXP, SEXP, SEXP, SEXP);
-extern SEXP rnng_unresolved(SEXP);
-extern SEXP rnng_unresolved2(SEXP);
-extern SEXP rnng_url_parse(SEXP);
-extern SEXP rnng_version(void);
-extern SEXP rnng_weakref_make(SEXP, SEXP);
-extern SEXP rnng_weakref_key(SEXP);
-extern SEXP rnng_weakref_value(SEXP);
-extern SEXP rnng_write_cert(SEXP, SEXP, SEXP);
-
 #endif
