@@ -103,7 +103,7 @@ nanotesterr(msg$data, "active")
 nanotest(identical(call_aio(n), n))
 nanotestaio(sraio <- n$send_aio(as.raw(0L), mode = "r", timeout = 500))
 nanotestaio(rraio <- n1$recv_aio(mode = 1L, timeout = 500))
-nanotestw(is_nul_byte(call_aio(rraio)$data))
+nanotestw(is_nul_byte(call_aio_(rraio)$data))
 nanotestaio(sraio <- n$send_aio(as.raw(1L), mode = "ra", timeout = 500))
 nanotestaio(rraio <- n1$recv_aio(mode = "raw", timeout = 500))
 nanotest(is.raw(call_aio(rraio)$data))
@@ -188,7 +188,7 @@ nanotesterr(req$opt("false", list()), "type")
 nanotestnano(ctx <- context(rep))
 nanotestp(ctx)
 nanotestaio(csaio <- req$send_aio(data.frame(), mode = "seria", timeout = 500))
-nanotest(is.integer(call_aio(csaio)$result))
+nanotest(is.integer(call_aio_(csaio)$result))
 nanotestaio(craio <- recv_aio(ctx, timeout = 500))
 nanotest(is.list(.subset2(call_aio(craio), "data")))
 nanotest(is.integer(req$send("context test", mode ="raw", block = 500)))
@@ -209,25 +209,26 @@ nanotestaio(err <- send_aio(ctx, "test"))
 nanotest(is_error_value(call_aio(err)$result))
 nanotest(is.integer(req$send(as.raw(1L), mode = 2L, block = 500)))
 nanotestaio(msg <- call_aio(recv_aio(ctx, mode = 8L, timeout = 500)))
-nanotestaio(saio <- send_aio(ctx, msg, mode = 2L, timeout = 500))
+nanotestaio(saio <- send_aio(ctx, msg[["value"]], mode = 2L, timeout = 500))
 nanotest(identical(req$recv(mode = 8L, block = 500), as.raw(1L)))
 nanotestaio(rek <- request(req$context, c(1+3i, 4+2i), send_mode = 2L, recv_mode = "complex", timeout = 500))
 nanotest(is.integer(reply(ctx, execute = identity, recv_mode = 3L, send_mode = "ra", timeout = 500)))
 nanotest(is.complex(call_aio(rek)[["data"]]))
 
-nanotest(is.pairlist(nxt <- nextmode(inhook = function(x) serialize(x, NULL), outhook = unserialize)))
+nanotest(is.list(nxt <- next_config(refhook = list(function(x) serialize(x, NULL), unserialize))))
 nanotest(length(nxt) == 2L)
 nanotest(is.function(nxt[[1L]]))
-nanotest(is.integer(req$send(list(new.env(), new.env()), mode = 3L, block = 500)))
-nanotest(is.environment(recv(rep, block = 500)[[1L]]))
-nanotest(is.pairlist(nextmode(NULL, NULL, mark = TRUE)))
-nanotestn(unlist(nextmode()))
+nanotest(is.integer(req$send(list(rep, ctx), mode = 3L, block = 500)))
+nanotest(typeof(recv(rep, block = 500)[[1L]]) == "externalptr")
+nanotest(is.list(next_config(NULL, mark = TRUE)))
+nanotestn(unlist(next_config()))
 
 nanotest(inherits(cv <- cv(), "conditionVariable"))
 nanotestp(cv)
-nanotest(!.until(cv, 10L))
-nanotest(!.until(cv, 10))
-nanotest(until(cv, 10L))
+nanotest(!until(cv, 10L))
+nanotest(!until(cv, 10))
+nanotest(!until_(cv, 10L))
+nanotest(!until_(cv, 10))
 nanotestz(cv_reset(cv))
 nanotestz(cv_value(cv))
 nanotestaio(cs <- request_signal(req$context, "test", send_mode = "next", cv = cv, timeout = 500))
@@ -241,6 +242,7 @@ nanotestaio(cs <- request(.context(req$socket), data = TRUE))
 nanotest(recv(ctxn, block = 500))
 nanotestz(send(ctxn, TRUE, mode = 3L, block = 500))
 nanotestz(reap(ctxn))
+nanotest(reap(ctxn) == 7L)
 nanotestz(pipe_notify(rep, cv, add = TRUE, remove = TRUE, flag = TRUE))
 nanotestz(pipe_notify(req$socket, cv = cv, add = FALSE, remove = TRUE, flag = FALSE))
 nanotesterr(request_signal(err, "test", cv = cv), "valid")
@@ -248,7 +250,9 @@ nanotesterr(recv_aio_signal(err, cv = cv, timeout = 500))
 nanotesterr(request_signal(.context(req$socket), "test", cv = err), "valid")
 nanotesterr(recv_aio_signal(rep, "test", cv = err), "valid")
 nanotesterr(wait(err), "valid")
-nanotesterr(.until(err, 10), "valid")
+nanotesterr(wait_(err), "valid")
+nanotesterr(until(err, 10), "valid")
+nanotesterr(until_(err, 10), "valid")
 nanotesterr(cv_value(err), "valid")
 nanotesterr(cv_reset(err), "valid")
 nanotesterr(cv_signal(err), "valid")
@@ -268,7 +272,7 @@ nanotestnano(pub <- nano("pub", listen = "inproc://ps"))
 nanotestnano(sub <- nano("sub", dial = "inproc://ps", autostart = NA))
 nanotestxp(cv <- cv())
 nanotestxp(cv2 <- cv())
-nanotestz(pipe_notify(pub$socket, cv, cv2, add = TRUE, remove = TRUE, flag = FALSE))
+nanotestz(pipe_notify(pub$socket, cv, cv2, add = TRUE, remove = TRUE))
 nanotestnano(sub$opt(name = "sub:prefnew", value = FALSE))
 nanotest(!sub$opt(name = "sub:prefnew"))
 nanotesterr(sub$opt(name = "false", value = 100), "supported")
@@ -288,8 +292,12 @@ nanotestz(sub$close())
 nanotestz(pub$close())
 nanotest(wait(cv))
 nanotest(wait(cv2))
+nanotestxp(cv3 <- cv())
+nanotestxp(cv %~>% cv2 %~>% cv3)
 nanotestz(cv_signal(cv))
 nanotest(cv_value(cv) == 1L)
+nanotest(wait_(cv3))
+nanotestxp(cv %~>% cv3)
 
 nanotestnano(surv <- nano(protocol = "surveyor", listen = "inproc://sock1", dial = "inproc://sock2"))
 nanotestp(surv)
@@ -339,7 +347,7 @@ nanotest(is.integer(call_aio(haio)$status))
 haio <- ncurl_aio("https://trin.cam.ac.uk/", convert = FALSE, response = list("server"))
 nanotest(is.integer(call_aio(haio)$status))
 put1 <- ncurl_aio("http://postman-echo.com/put", method = "PUT", headers = c(Authorization = "Bearer token"), data = "test", response = c("date", "server"), timeout = 3000L)
-nanotest(is.integer(call_aio(put1)$status))
+nanotest(is.integer(call_aio_(put1)$status))
 nanotestnn(put1$headers)
 nanotestnn(put1$data)
 put2 <- ncurl_aio("http://postman-echo.com/put", method = "PUT", headers = list(Authorization = "Bearer token"), response = list("date", "server"), timeout = 3000L)
@@ -411,7 +419,7 @@ if (length(s)) {
   nanotestn(stop_aio(ss))
   nanotest(is.integer(send(s, 12.56, mode = "raw", block = 500L)))
   nanotestaio(sr <- recv_aio_signal(s, mode = "double", timeout = 500L, cv = cv))
-  nanotestw(length(call_aio(sr)[["data"]]))
+  nanotestw(length(call_aio_(sr)[["data"]]))
   nanotest(cv_value(cv) > 0L)
   nanotest(wait(cv))
   nanotest(is.character(opt(s, "ws:request-headers")))
@@ -465,7 +473,7 @@ nanotest(!.unresolved(fakectx))
 nanotesterr(request(fakectx, data = "test"), "valid Context")
 nanotesterr(subscribe(fakectx, NULL), "valid")
 nanotesterr(close(fakectx), "valid Context")
-nanotesterr(reap(fakectx), "valid")
+nanotest(reap(fakectx) == 3L)
 fakestream <- `class<-`("test", "nanoStream")
 nanotestp(fakestream)
 fakesession <- `class<-`("test", "ncurlSession")
@@ -489,6 +497,7 @@ unres <- `class<-`(NA, "unresolvedValue")
 nanotest(!unresolved(unres))
 nanotestp(unres)
 nanotest(identical(call_aio("a"), "a"))
+nanotest(identical(call_aio_("a"), "a"))
 nanotestn(stop_aio("a"))
 
 nanotest(sha256("test") == "9f86d081884c7d659a2feaa0c55ad015a3bf4f1b2b0b822cd15d6c15b0f00a08")
@@ -511,14 +520,6 @@ nanotest(is.integer(base64dec(base64enc(c(1L, 2L)), convert = NA)))
 nanotesterr(base64dec("__"), "not valid base64")
 nanotesterr(base64dec(404), "not valid base64")
 
-k <- new.env()
-v <- "value"
-nanotest(typeof(w <- weakref(k, v)) == "weakref")
-nanotest(identical(weakref_key(w), k))
-nanotest(identical(weakref_value(w), v))
-rm(v)
-nanotest(weakref_value(w) == "value")
-
 file <- tempfile()
 pem <- "-----BEGIN CERTIFICATE----- -----END CERTIFICATE-----"
 cat(pem, file = file)
